#!/bin/sh
#
# see ./qthsh-test.sh
#
cat > temprun.cpp << END
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#define pi M_PI
#define sign(x) ((x>0)-(x<0))
#define csc(x) (1/sin(x))
#define csch(x) (1/sinh(x))
#define sec(x) (1/cos(x))
#define sech(x) (1/cosh(x))
#define cot(x) (1/tan(x))
#define coth(x) (1/tanh(x))
double f(double x) { return $1; }
void qthsh(double (*f)(double), double a, double b, double n, double eps) {
  double c = (a+b)/2;
  double d = (b-a)/2;
  double s = f(c);
  double e, v, h = 2;
  int k = 0;
  int ev = 1;
  do {
    double p = 0, q, fp = 0, fm = 0, t, eh;
    h /= 2;
    eh = exp(h);
    t = eh;
    if (k > 0)
      eh *= eh;
    do {
      double u = exp(1/t-t);      // = exp(-2*sinh(j*h)) = 1/exp(sinh(j*h))^2
      double r = 2*u/(1+u);       // = 1 - tanh(sinh(j*h))
      double w = (t+1/t)*r/(1+u); // = cosh(j*h)/cosh(sinh(j*h))^2
      double x = d*r;
      if (a+x > a) {              // if too close to a then reuse previous fp
        double y = f(a+x); ++ev;
        if (isfinite(y))
          fp = y;                 // if f(x) is finite, add to local sum
      }
      if (b-x < b) {              // if too close to b then reuse previous fm
        double y = f(b-x); ++ev;
        if (isfinite(y))
          fm = y;                 // if f(x) is finite, add to local sum
      }
      q = w*(fp+fm);
      p += q;
      t *= eh;
    } while (fabs(q) > eps*fabs(p));
    v = 2*s;
    s += p;
    ++k;
  } while (fabs(v-s) > 10*eps*fabs(s) && k <= n);
  e = fabs(v-s)/(fabs(s)+eps);
  s *= d*h;
  printf("%.16g,%d,%d,%g,%g\n", s, k-1, ev, e, fabs(s-($5)));
}
int main() {
  qthsh(f, $2, $3, 6, $4);
}
END
c++ temprun.cpp && ./a.out
