#!/bin/sh
#
# see ./quad-test.sh
#
cat > temprun.cpp << END
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#define pi M_PI
#define sign(x) ((x>0)-(x<0))
#define csc(x) (1/sin(x))
#define csch(x) (1/sinh(x))
#define sec(x) (1/cos(x))
#define sech(x) (1/cosh(x))
#define cot(x) (1/tan(x))
#define coth(x) (1/tanh(x))
double f(double x) { return $1; }
void quad(double (*f)(double), double a, double b, double n, double eps) {
  double c = 0, d = 1, s, sign = 1, e, v, h = 2;
  int ev, k = 0, mode = 0; // Tanh-Sinh = 0, Exp-Sinh = 1, Sinh-Sinh = 2
  if (b < a) { // swap bounds
    v = b;
    b = a;
    a = v;
    sign = -1;
  }
  if (isfinite(a) && isfinite(b)) {
    c = (a+b)/2;
    d = (b-a)/2;
    v = c;
  }
  else if (isfinite(a)) {
    mode = 1; // Exp-Sinh
    c = a;
    v = a+1;
  }
  else if (isfinite(b)) {
    mode = 1; // Exp-Sinh
    c = b;
    v = b-1;
    d = -d;
    sign = -sign;
  }
  else {
    mode = 2; // Sinh-Sinh
    v = 0;
  }
  s = f(v); ev = 1;
  do {
    double p = 0, q, fp = 0, fm = 0, t, eh;
    h /= 2;
    t = eh = exp(h);
    if (k > 0)
      eh *= eh;
    if (mode == 0) {                // Tanh-Sinh
      do {
        double u = exp(1/t-t);      // = exp(-2*sinh(j*h)) = 1/exp(sinh(j*h))^2
        double r = 2*u/(1+u);       // = 1 - tanh(sinh(j*h))
        double w = (t+1/t)*r/(1+u); // = cosh(j*h)/cosh(sinh(j*h))^2
        double x = d*r;
        if (a+x > a) {              // if too close to a then reuse previous fp
          double y = f(a+x); ++ev;
          if (isfinite(y))
            fp = y;                 // if f(x) is finite, add to local sum
        }
        if (b-x < b) {              // if too close to b then reuse previous fm
          double y = f(b-x); ++ev;
          if (isfinite(y))
            fm = y;                 // if f(x) is finite, add to local sum
        }
        q = w*(fp+fm);
        p += q;
        t *= eh;
      } while (fabs(q) > eps*fabs(p));
    }
    else {
      t /= 2;
      do {
        double r = exp(t-.25/t); // = exp(sinh(j*h))
        double x, y, w = r;
        q = 0;
        if (mode == 1) {         // Exp-Sinh
          x = c + d/r;
          if (x == c)            // if x hit the finite endpoint then break
            break;
          y = f(x); ++ev;
          if (isfinite(y))       // if f(x) is finite, add to local sum
            q += y/w;
        }
        else {                   // Sinh-Sinh
          r = (r-1/r)/2;         // = sinh(sinh(j*h))
          w = (w+1/w)/2;         // = cosh(sinh(j*h))
          x = c - d*r;
          y = f(x); ++ev;
          if (isfinite(y))       // if f(x) is finite, add to local sum
            q += y*w;
        }
        x = c + d*r;
        y = f(x); ++ev;
        if (isfinite(y))         // if f(x) is finite, add to local sum
          q += y*w;
        q *= t+.25/t;            // q *= cosh(j*h)
        p += q;
        t *= eh;
      } while (fabs(q) > eps*fabs(p));
    }
    v = 2*s;
    s += p;
    ++k;
  } while (fabs(v-s) > 10*eps*fabs(s) && k <= n);
  e = fabs(v-s)/(fabs(s)+eps);
  s *= sign*d*h;
  printf("%.16g,%d,%d,%g,%g\n", s, k-1, ev, e, fabs(s-($5)));
}
int main() {
  quad(f, $2, $3, 6, $4);
}
END
c++ temprun.cpp && ./a.out
