#!/bin/sh
#
# see ./wo34s-test.sh
#
cat > temprun.cpp << END
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#define pi M_PI
#define sign(x) ((x>0)-(x<0))
#define csc(x) (1/sin(x))
#define csch(x) (1/sinh(x))
#define sec(x) (1/cos(x))
#define sech(x) (1/cosh(x))
#define cot(x) (1/tan(x))
#define coth(x) (1/tanh(x))
double f(double x) { return $1; }
void wp34s(double (*f)(double), double a, double b, double n, double eps) {
  const double thr = 10*sqrt(eps);
  double c = (a+b)/2;
  double d = (b-a)/2;
  double s = f(c);
  double fp = 0, fm = 0;
  double e, v, h = 2;
  double p;
  int k = 0;
  int ev = 1;
  double tmax = log(2/M_PI * log((d < 1 ? 2*d : 2) / eps));
  do {
    double q, t;
    int j;
    v = s*d*M_PI/2*h;
    p = 0;
    h /= 2;
    t = h;
    j = 1;
    do {
      double ch = cosh(t);
      double ecs = cosh(M_PI/2 * sqrt(ch*ch - 1));
      double w = 1/(ecs*ecs);
      double r = sqrt(ecs*ecs - 1)/ecs;
      double x, y;
      x = d*r;
      if (c+x > a) {
        y = f(c+x), ++ev;
        if (isfinite(y))
          fp = y;
      }
      if (c-x < b) {
        y = f(c-x), ++ev;
        if (isfinite(y))
          fm = y;
      }
      q = ch*w*(fp+fm);
      p += q;
      j += 1+(k>0);
      t = j*h;
    } while (t <= tmax && fabs(q) > eps*fabs(p));
    s += p;
    ++k;
  } while (s && fabs(2*fabs(p) - fabs(s)) >= fabs(thr*s) && k <= n);
  s *= d*M_PI/2*h;
  e = fabs(v-s)/(fabs(s)+eps);
  printf("%.16g,%d,%d,%g,%g\n", s, k-1, ev, e, fabs(s-($5)));
}
int main() {
  wp34s(f, $2, $3, 6, $4);
}
END
c++ temprun.cpp && ./a.out

